@@ -106,6 +106,27 @@ equal(Type t, Ftor f) {
   return equal_t<Type, Ftor>(t, f);
 }
 
+template <typename Type, typename Ftor>
+struct equal_ptr_t {
+  typedef bool result_type;
+
+  equal_ptr_t(Type* t, Ftor f) : m_t(t), m_f(f) {}
+
+  template <typename Arg>
+  bool operator () (const Arg& a) {
+    return *m_t == *m_f(a);
+  }
+
+  Type* m_t;
+  Ftor  m_f;
+};
+
+template <typename Type, typename Ftor>
+inline equal_ptr_t<Type, Ftor>
+equal_ptr(Type* t, Ftor f) {
+  return equal_ptr_t<Type, Ftor>(t, f);
+}
+
 template <typename Type, typename Ftor>
 struct not_equal_t {
   typedef bool result_type;
@@ -148,6 +169,24 @@ less(Type t, Ftor f) {
   return less_t<Type, Ftor>(t, f);
 }
 
+template <typename FtorA, typename FtorB>
+struct less2_t : public std::binary_function<typename FtorA::argument_type, typename FtorB::argument_type, bool> {
+  less2_t(FtorA f_a, FtorB f_b) : m_f_a(f_a), m_f_b(f_b) {}
+
+  bool operator () (typename FtorA::argument_type a, typename FtorB::argument_type b) {
+    return m_f_a(a) < m_f_b(b);
+  }
+
+  FtorA m_f_a;
+  FtorB m_f_b;
+};
+
+template <typename FtorA, typename FtorB>
+inline less2_t<FtorA, FtorB>
+less2(FtorA f_a, FtorB f_b) {
+  return less2_t<FtorA,FtorB>(f_a,f_b);
+}
+
 template <typename Type, typename Ftor>
 struct _greater {
   typedef bool result_type;
@@ -329,7 +368,7 @@ public:
     m_op(op), m_value(v) {}
 
   typename Operation::result_type
-  operator () (const argument_type arg) {
+  operator () (argument_type arg) {
     return m_op(arg, m_value);
   }
 
@@ -421,13 +460,17 @@ template <typename Object, typename Ret, typename Arg1, typename Arg2>
 class mem_fun2 : public std::binary_function<Arg1, Arg2, Ret> {
 public:
   typedef Ret (Object::*Function)(Arg1, Arg2);
+  typedef Object object_type;
 
   mem_fun2() : m_object(NULL) {}
   mem_fun2(Object* o, Function f) : m_object(o), m_function(f) {}
 
-  bool is_valid() const { return m_object; }
+  bool         is_valid() const { return m_object; }
 
-  Ret operator () (Arg1 a1, Arg2 a2) { return (m_object->*m_function)(a1, a2); }
+  object_type*       object() { return m_object; }
+  const object_type* object() const { return m_object; }
+
+  Ret          operator () (Arg1 a1, Arg2 a2) { return (m_object->*m_function)(a1, a2); }
   
 private:
   Object* m_object;
