@@ -37,9 +37,16 @@
 #include "config.h"
 
 #include <unistd.h>
+#include <sys/stat.h>
+#include <torrent/connection_manager.h>
 
 #include "core/manager.h"
+#include "core/download_store.h"
+#include "core/view_manager.h"
+#include "core/scheduler.h"
+
 #include "display/canvas.h"
+#include "display/client_info.h"
 #include "display/window.h"
 #include "display/manager.h"
 #include "input/manager.h"
@@ -56,7 +63,6 @@ Control::Control() :
   m_shutdownQuick(false),
 
   m_ui(new ui::Root()),
-  m_core(new core::Manager()),
   m_display(new display::Manager()),
   m_input(new input::Manager()),
   m_inputStdin(new input::InputEvent(STDIN_FILENO)),
@@ -64,8 +70,14 @@ Control::Control() :
   m_commandScheduler(new CommandScheduler()),
   m_variables(new utils::VariableMap()),
 
+  m_clientInfo(new display::ClientInfo),
+
   m_tick(0) {
 
+  m_core        = new core::Manager();
+  m_viewManager = new core::ViewManager(m_core->download_list());
+  m_scheduler   = new core::Scheduler(m_core->download_list());
+
   m_inputStdin->slot_pressed(sigc::mem_fun(m_input, &input::Manager::pressed));
 
   m_taskShutdown.set_slot(rak::mem_fn(this, &Control::handle_shutdown));
@@ -84,6 +96,8 @@ Control::~Control() {
   delete m_ui;
   delete m_display;
   delete m_core;
+
+  delete m_clientInfo;
 }
 
 void
@@ -97,7 +111,10 @@ Control::initialize() {
 
   m_core->initialize_second();
   m_core->listen_open();
-  m_core->download_store().enable(m_variables->get_string("session_lock") == "yes");
+  m_core->download_store()->enable(m_variables->get_value("session_lock"));
+
+  m_core->set_hashing_view(*m_viewManager->find_throw("hashing"));
+  m_scheduler->set_view(*m_viewManager->find_throw("scheduler"));
 
   m_ui->init(this);
 
@@ -110,7 +127,7 @@ Control::cleanup() {
 
   m_inputStdin->remove(m_core->get_poll_manager()->get_torrent_poll());
 
-  m_core->download_store().disable();
+  m_core->download_store()->disable();
 
   m_ui->cleanup();
   m_core->cleanup();
@@ -124,7 +141,7 @@ Control::cleanup() {
 void
 Control::handle_shutdown() {
   if (!m_shutdownQuick) {
-    torrent::listen_close();
+    torrent::connection_manager()->listen_close();
     m_core->shutdown(false);
 
     if (!m_taskShutdown.is_queued())
@@ -137,3 +154,16 @@ Control::handle_shutdown() {
   m_shutdownQuick = true;
   m_shutdownReceived = false;
 }
+
+void
+Control::set_umask(mode_t m) {
+  ::umask(m);
+
+  m_umask = m;
+}
+
+void
+Control::set_working_directory(const std::string& dir) {
+  if (::chdir(dir.c_str()) != 0)
+    throw torrent::input_error("Could not change working directory.");
+}
