@@ -68,7 +68,7 @@ Download::Download(download_type d) :
   m_download.signal_chunk_failed(sigc::mem_fun(*this, &Download::receive_chunk_failed));
 
   m_variables.insert("connection_current", new utils::VariableStringSlot(rak::mem_fn(this, &Download::connection_current),
-									 rak::mem_fn(this, &Download::set_connection_current)));
+                                                                         rak::mem_fn(this, &Download::set_connection_current)));
 
   m_variables.insert("connection_leech",   new utils::VariableAny(connection_type_to_string(download_type::CONNECTION_LEECH)));
   m_variables.insert("connection_seed",    new utils::VariableAny(connection_type_to_string(download_type::CONNECTION_SEED)));
@@ -89,17 +89,19 @@ Download::Download(download_type d) :
   // resume/pause.
   m_variables.insert("state_changed",      new utils::VariableObject(bencode(), "rtorrent", "state_changed", torrent::Object::TYPE_VALUE));
 
-  m_variables.insert("directory",          new utils::VariableStringSlot(rak::mem_fn(&m_fileList, &torrent::FileList::root_dir),
-									 rak::mem_fn(this, &Download::set_root_directory)));
+  m_variables.insert("directory",          new utils::VariableStringSlot(rak::mem_fn(&m_fileList, &torrent::FileList::root_dir), rak::mem_fn(this, &Download::set_root_directory)));
 
-  m_variables.insert("min_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_min),
-									rak::mem_fn(&m_download, &download_type::set_peers_min)));
-  m_variables.insert("max_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_max),
-									rak::mem_fn(&m_download, &download_type::set_peers_max)));
-  m_variables.insert("max_uploads",        new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::uploads_max),
-									rak::mem_fn(&m_download, &download_type::set_uploads_max)));
-  m_variables.insert("priority",           new utils::VariableValueSlot(rak::mem_fn(this, &Download::priority),
-									rak::mem_fn(this, &Download::set_priority)));
+  m_variables.insert("min_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_min), rak::mem_fn(&m_download, &download_type::set_peers_min)));
+  m_variables.insert("max_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_max), rak::mem_fn(&m_download, &download_type::set_peers_max)));
+  m_variables.insert("max_uploads",        new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::uploads_max), rak::mem_fn(&m_download, &download_type::set_uploads_max)));
+  m_variables.insert("max_chunks_queued",  new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::max_chunks_queued), rak::mem_fn(&m_download, &download_type::set_max_chunks_queued)));
+
+  m_variables.insert("timeout_sync",       new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::timeout_sync), rak::mem_fn(&m_download, &download_type::set_timeout_sync)));
+  m_variables.insert("timeout_safe_sync",  new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::timeout_safe_sync), rak::mem_fn(&m_download, &download_type::set_timeout_safe_sync)));
+
+  m_variables.insert("priority",           new utils::VariableValueSlot(rak::mem_fn(this, &Download::priority), rak::mem_fn(this, &Download::set_priority)));
+
+  m_variables.insert("ignore_commands",    new utils::VariableObject(bencode(), "rtorrent", "ignore_commands", torrent::Object::TYPE_VALUE));
 }
 
 Download::~Download() {
@@ -120,14 +122,14 @@ Download::enable_udp_trackers(bool state) {
   for (int i = 0, last = tl.size(); i < last; ++i)
     if (tl.get(i).tracker_type() == torrent::Tracker::TRACKER_UDP)
       if (state)
-	tl.get(i).enable();
+        tl.get(i).enable();
       else
-	tl.get(i).disable();
+        tl.get(i).disable();
 }
 
 uint32_t
 Download::priority() {
-  return bencode()->get_key("rtorrent").get_key("priority").as_value();
+  return bencode()->get_key("rtorrent").get_key_value("priority");
 }
 
 void
