@@ -55,10 +55,10 @@ public:
   typedef download_type::ConnectionType connection_type;
   typedef utils::VariableMap            variable_map_type;
 
-  static const int64_t variable_hashing_stopped = 0;
-  static const int64_t variable_hashing_initial = 1;
-  static const int64_t variable_hashing_last    = 2;
-  static const int64_t variable_hashing_rehash  = 3;
+  static const int variable_hashing_stopped = 0;
+  static const int variable_hashing_initial = 1;
+  static const int variable_hashing_last    = 2;
+  static const int variable_hashing_rehash  = 3;
 
   Download(download_type d);
   ~Download();
@@ -66,6 +66,8 @@ public:
   bool                is_open() const                          { return m_download.is_open(); }
   bool                is_active() const                        { return m_download.is_active(); }
   bool                is_done() const                          { return m_download.chunks_done() == m_download.chunks_total(); }
+  bool                is_downloading() const                   { return is_active() && !is_done(); }
+  bool                is_seeding() const                       { return is_active() && is_done(); }
 
   // FIXME: Fixed a bug in libtorrent that caused is_hash_checked to
   // return true when the torrent is closed. Remove this redundant
@@ -76,8 +78,9 @@ public:
   bool                is_hash_failed() const                   { return m_hashFailed; }
   void                set_hash_failed(bool v)                  { m_hashFailed = v; }
 
-  variable_map_type*  variable()                               { return &m_variables; }
-  std::string         variable_string(const std::string& key)  { return m_variables.get_string(key); }
+  variable_map_type*  variable()                                     { return &m_variables; }
+  int64_t             variable_value(const std::string& key) const   { return m_variables.get_value(key); }
+  const std::string&  variable_string(const std::string& key) const  { return m_variables.get_string(key); }
 
   download_type*       download()                              { return &m_download; }
   const download_type* download() const                        { return &m_download; }
