@@ -44,6 +44,7 @@
 #include <torrent/object.h>
 #include <torrent/exceptions.h>
 #include <torrent/rate.h>
+#include <torrent/resume.h>
 
 #include "utils/variable_generic.h"
 
@@ -112,9 +113,10 @@ DownloadFactory::receive_load() {
     m_variables.set("tied_to_file", (int64_t)false);
 
   } else {
-    m_stream = new std::fstream(rak::path_expand(m_uri).c_str(), std::ios::in | std::ios::binary);
+    std::fstream* stream = new std::fstream(rak::path_expand(m_uri).c_str(), std::ios::in | std::ios::binary);
+    m_stream = stream;
 
-    if (m_stream->good())
+    if (stream->is_open())
       receive_loaded();
     else
       receive_failed("Could not open file");
@@ -174,6 +176,18 @@ DownloadFactory::receive_success() {
   download->variable()->set("max_peers",        control->variable()->get("max_peers"));
   download->variable()->set("max_uploads",      control->variable()->get("max_uploads"));
 
+  if (!control->variable()->get_value("timeout_sync") != 0)
+    download->variable()->set("timeout_sync", control->variable()->get("timeout_sync"));
+
+  if (!control->variable()->get_value("timeout_safe_sync") != 0)
+    download->variable()->set("timeout_safe_sync", control->variable()->get("timeout_safe_sync"));
+
+  if (!control->variable()->get_value("max_chunks_queued") != 0)
+    download->variable()->set("max_chunks_queued", control->variable()->get("max_chunks_queued"));
+  else
+    // Temporary until we fix the library.
+    download->variable()->set("max_chunks_queued", (512 << 20) / download->download()->chunks_size());
+
   if (!control->variable()->get_value("use_udp_trackers"))
     download->enable_udp_trackers(false);
 
@@ -185,6 +199,13 @@ DownloadFactory::receive_success() {
   if (!m_session && m_variables.get("tied_to_file").as_value())
     download->variable()->set("tied_to_file", m_uri);
 
+  torrent::Object& resumeObject = root->has_key_map("libtorrent_resume")
+    ? root->get_key("libtorrent_resume")
+    : root->insert_key("libtorrent_resume", torrent::Object(torrent::Object::TYPE_MAP));
+
+  torrent::resume_load_addresses(*download->download(), resumeObject);
+  torrent::resume_load_file_priorities(*download->download(), resumeObject);
+
   // The action of inserting might cause the torrent to be
   // opened/started or such. Figure out a nicer way of handling this.
   if (m_manager->download_list()->insert(download) == m_manager->download_list()->end()) {
@@ -201,13 +222,13 @@ DownloadFactory::receive_success() {
     // This torrent was queued for hashing or hashing when the session
     // file was saved. Or it was in a started state.
     if (download->variable()->get_value("hashing") != Download::variable_hashing_stopped ||
-	download->variable()->get_value("state") != 0)
+        download->variable()->get_value("state") != 0)
       m_manager->download_list()->resume(download);
 
   } else {
     // Use the state thingie here, move below.
     if (m_start)
-      m_manager->download_list()->start(download);
+      m_manager->download_list()->start_normal(download);
 
     m_manager->download_store()->save(download);
   }
@@ -231,12 +252,12 @@ DownloadFactory::receive_failed(const std::string& msg) {
 
 void
 DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorrent) {
-  if (!rtorrent->has_key_value("state") || rtorrent->get_key("state").as_value() > 1) {
+  if (!rtorrent->has_key_value("state") || rtorrent->get_key_value("state") > 1) {
     rtorrent->insert_key("state", (int64_t)m_start);
     rtorrent->insert_key("state_changed", cachedTime.seconds());
 
   } else if (!rtorrent->has_key_value("state_changed") ||
-	     rtorrent->get_key("state_changed").as_value() > cachedTime.seconds() || rtorrent->get_key("state_changed").as_value() == 0) {
+             rtorrent->get_key_value("state_changed") > cachedTime.seconds() || rtorrent->get_key_value("state_changed") == 0) {
     rtorrent->insert_key("state_changed", cachedTime.seconds());
   }
 
@@ -244,26 +265,32 @@ DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorre
     rtorrent->insert_key("complete", (int64_t)0);
 
   if (!rtorrent->has_key_value("hashing"))
-    rtorrent->insert_key("hashing", Download::variable_hashing_stopped);
+    rtorrent->insert_key("hashing", (int64_t)Download::variable_hashing_stopped);
 
   if (!rtorrent->has_key_string("tied_to_file"))
     rtorrent->insert_key("tied_to_file", std::string());
 
   if (rtorrent->has_key_value("priority"))
-    download->variable()->set("priority", rtorrent->get_key("priority").as_value() % 4);
+    download->variable()->set("priority", rtorrent->get_key_value("priority") % 4);
   else
     download->variable()->set("priority", (int64_t)2);
 
   if (rtorrent->has_key_value("key")) {
-    download->tracker_list()->set_key(rtorrent->get_key("key").as_value());
+    download->tracker_list()->set_key(rtorrent->get_key_value("key"));
 
   } else {
-    download->tracker_list()->set_key(rand() % (std::numeric_limits<uint32_t>::max() - 1) + 1);
+    download->tracker_list()->set_key(random() % (std::numeric_limits<uint32_t>::max() - 1) + 1);
     rtorrent->insert_key("key", download->tracker_list()->key());
   }
 
   if (rtorrent->has_key_value("total_uploaded"))
-    download->download()->up_rate()->set_total(rtorrent->get_key("total_uploaded").as_value());
+    download->download()->up_rate()->set_total(rtorrent->get_key_value("total_uploaded"));
+
+  if (rtorrent->has_key_value("chunks_done"))
+    download->download()->set_chunks_done(std::min<uint32_t>(rtorrent->get_key_value("chunks_done"), download->download()->chunks_total()));
+
+  if (!rtorrent->has_key_value("ignore_commands"))
+    rtorrent->insert_key("ignore_commands", (int64_t)0);
 }
 
 }
