@@ -42,6 +42,7 @@
 #include <torrent/exceptions.h>
 #include <torrent/object.h>
 #include <torrent/object_stream.h>
+#include <torrent/resume.h>
 #include <torrent/torrent.h>
 
 #include "rak/functional.h"
@@ -160,19 +161,17 @@ DownloadList::erase(iterator itr) {
   return base_type::erase(itr);
 }
 
-// void
-// DownloadList::save(Download* d) {
-  
-// }
-
-void
+bool
 DownloadList::open(Download* download) {
   try {
 
     open_throw(download);
 
+    return true;
+
   } catch (torrent::local_error& e) {
     control->core()->push_log(e.what());
+    return false;
   }
 }
 
@@ -203,14 +202,15 @@ void
 DownloadList::close_throw(Download* download) {
   check_contains(download);
 
+  // When pause gets called it will clear the initial hash check state
+  // and set hash failed. This should ensure hashing doesn't restart
+  // until resume gets called.
+  pause(download);
+
+  // Check for is_open after pause due to hashing.
   if (!download->is_open())
     return;
 
-  // When pause gets called it will clear the initial hash checking. 
-
-  if (download->is_active())
-    pause(download);
-  
   // Save the torrent on close, this covers shutdown and if a torrent
   // is manually closed which would clear the progress data. For
   // better crash protection, save regulary in addition to this.
@@ -220,46 +220,47 @@ DownloadList::close_throw(Download* download) {
   // Reconsider this save. Should be done explicitly when shutting down.
   //control->core()->download_store()->save(download);
 
-  // FIXME: Urgh, need to do something sane when closing a download
-  // that has been queued for hashing. ATM just close the torrent and
-  // call the hash_removed slots. This might open and restart the
-  // hashing of this torrent.
-
-//   if (download->is_hash_checking()) {
-//     download->download()->close();
-
-//     // Hash removed slot must be called after close as we can't atm
-//     // stop already started hash checks except through close.
-//     std::for_each(slot_map_hash_removed().begin(), slot_map_hash_removed().end(), download_list_call(download));
-
-//   } else {
-//     download->download()->close();
-//   }
-
-  // But this isn't correct either, i think... ATM do the borking
-  // thing, the hash queue won't be updated. Need to properly handle
-  // erasing a download.
-
-  // Propably should do its own thingie in erase rather than calling
-  // close.
-
   download->download()->close();
 
+  if (!download->is_hash_failed() && download->variable()->get_value("hashing") != Download::variable_hashing_stopped)
+    throw torrent::client_error("DownloadList::close_throw(...) called but we're going into a hashing loop.");
+
   std::for_each(slot_map_hash_removed().begin(), slot_map_hash_removed().end(), download_list_call(download));
   std::for_each(slot_map_close().begin(), slot_map_close().end(), download_list_call(download));
 }
 
 void
-DownloadList::start(Download* download) {
+DownloadList::start_normal(Download* download) {
   check_contains(download);
 
+  // Clear hash failed as we're doing a manual start and want to try
+  // hashing again.
+  download->set_hash_failed(false);
   download->variable()->set("state", (int64_t)1);
 
   resume(download);
 }
 
+bool
+DownloadList::start_try(Download* download) {
+  check_contains(download);
+
+  // Also don't start if the state is one of those that indicate we
+  // were manually stopped?
+
+  if (download->is_hash_failed() || download->variable()->get_value("ignore_commands") != 0)
+    return false;
+
+  // Don't clear the hash failed as this function is used by scripts,
+  // etc.
+  download->variable()->set("state", (int64_t)1);
+
+  resume(download);
+  return true;
+}
+
 void
-DownloadList::stop(Download* download) {
+DownloadList::stop_normal(Download* download) {
   check_contains(download);
 
   download->variable()->set("state", (int64_t)0);
@@ -267,6 +268,19 @@ DownloadList::stop(Download* download) {
   pause(download);
 }
 
+bool
+DownloadList::stop_try(Download* download) {
+  check_contains(download);
+
+  if (download->variable()->get_value("ignore_commands") != 0)
+    return false;
+
+  download->variable()->set("state", (int64_t)0);
+
+  pause(download);
+  return true;
+}
+
 void
 DownloadList::resume(Download* download) {
   check_contains(download);
@@ -282,21 +296,22 @@ DownloadList::resume(Download* download) {
     // It is also assumed the is_hash_checked flag gets cleared when
     // 'hashing' was set.
     if (!download->is_hash_checked()) {
-      download->set_hash_failed(false);
+      // If the hash failed flag wasn't cleared then hashing won't be
+      // initiated.
+      if (download->is_hash_failed())
+        return;
 
-      // Set 'hashing' to started if hashing wasn't started, else keep
-      // the old value.
       if (download->variable()->get_value("hashing") == Download::variable_hashing_stopped)
-	download->variable()->set("hashing", Download::variable_hashing_initial);
+        download->variable()->set("hashing", Download::variable_hashing_initial);
 
       std::for_each(slot_map_hash_queued().begin(), slot_map_hash_queued().end(), download_list_call(download));
       return;
     }
 
-    download->variable()->set("state_changed", cachedTime.seconds());
-
     open_throw(download);
 
+    download->variable()->set("state_changed", cachedTime.seconds());
+
     if (download->is_done()) {
       download->set_connection_type(download->variable()->get_string("connection_seed"));
     } else {
@@ -306,7 +321,7 @@ DownloadList::resume(Download* download) {
       // on non-complete downloads after a crash. This shouldn't be
       // needed, but for some reason linux 2.6 is very lazy about
       // updating mtime.
-      download->download()->hash_resume_clear();
+      torrent::resume_clear_progress(*download->download(), download->download()->bencode()->get_key("libtorrent_resume"));
     }
 
     // Update the priority to ensure it has the correct
@@ -327,9 +342,11 @@ DownloadList::pause(Download* download) {
 
   try {
 
-    // Don't stop if we're doing the final hashing.
-    if (download->variable()->get_value("hashing") == Download::variable_hashing_initial) {
-      download->variable()->set("hashing", Download::variable_hashing_stopped);
+    // Always clear hashing on pause. When a hashing request is added,
+    // it should have cleared the hash resume data.
+    if (download->variable()->get_value("hashing") != Download::variable_hashing_stopped) {
+      download->download()->hash_stop();
+      download->variable()->set_value("hashing", Download::variable_hashing_stopped);
 
       std::for_each(slot_map_hash_removed().begin(), slot_map_hash_removed().end(), download_list_call(download));
     }
@@ -338,7 +355,7 @@ DownloadList::pause(Download* download) {
       return;
 
     download->download()->stop();
-    download->download()->hash_resume_save();
+    torrent::resume_save_progress(*download->download(), download->download()->bencode()->get_key("libtorrent_resume"));
     
     std::for_each(slot_map_stop().begin(), slot_map_stop().end(), download_list_call(download));
 
@@ -359,8 +376,10 @@ DownloadList::check_hash(Download* download) {
 
   try {
 
-    download->variable()->set("hashing", Download::variable_hashing_rehash);
-    hash_clear(download);
+    if (download->variable()->get_value("hashing") != Download::variable_hashing_stopped)
+      return;
+
+    hash_queue(download, Download::variable_hashing_rehash);
 
   } catch (torrent::local_error& e) {
     control->core()->push_log(e.what());
@@ -374,9 +393,16 @@ DownloadList::hash_done(Download* download) {
   if (!download->is_open())
     throw torrent::client_error("DownloadList::hash_done(...) !download->is_open().");
 
-  if (!download->is_hash_checked() || download->is_hash_checking() || download->is_active())
+  if (download->is_hash_checking() || download->is_active())
     throw torrent::client_error("DownloadList::hash_done(...) download in invalid state.");
 
+  if (!download->is_hash_checked()) {
+    download->set_hash_failed(true);
+    
+    std::for_each(slot_map_hash_done().begin(), slot_map_hash_done().end(), download_list_call(download));
+    return;
+  }
+
   // Need to find some sane conditional here. Can we check the total
   // downloaded to ensure something was transferred, thus we didn't
   // just hash an already completed torrent with lacking session data?
@@ -387,19 +413,24 @@ DownloadList::hash_done(Download* download) {
   // ReiserFS bug with >4GB files.
 
   int64_t hashing = download->variable()->get_value("hashing");
-  download->variable()->set("hashing", Download::variable_hashing_stopped);
+  download->variable()->set_value("hashing", Download::variable_hashing_stopped);
 
   switch (hashing) {
   case Download::variable_hashing_initial:
   case Download::variable_hashing_rehash:
     // Normal re/hashing.
 
-    if (download->is_done())
-      download->variable()->set("complete", (int64_t)1);
-    
+    // If the download was previously completed but the files were
+    // f.ex deleted, then we clear the state and complete.
+    if (download->variable()->get_value("complete") && !download->is_done()) {
+      download->variable()->set("state", (int64_t)0);
+      download->set_message("Download registered as completed, but hash check returned unfinished chunks.");
+    }
+
     // Save resume data so we update time-stamps and priorities if
-    // they were invalid/changed when loading.
-    download->download()->hash_resume_save();
+    // they were invalid/changed while loading/hashing.
+    download->variable()->set("complete", (int64_t)download->is_done());
+    torrent::resume_save_progress(*download->download(), download->download()->bencode()->get_key("libtorrent_resume"));
 
     if (download->variable()->get_value("state") == 1)
       resume(download);
@@ -426,17 +457,17 @@ DownloadList::hash_done(Download* download) {
 }
 
 void
-DownloadList::hash_clear(Download* download) {
+DownloadList::hash_queue(Download* download, int type) {
   check_contains(download);
 
-  // Set hash failed so close_throw won't cause the hash queue to
-  // reopen this download.
-  download->set_hash_failed(true);
+  if (download->variable()->get_value("hashing") != Download::variable_hashing_stopped)
+    throw torrent::client_error("DownloadList::hash_queue(...) hashing already queued.");
 
   close_throw(download);
-  download->download()->hash_resume_clear();
+  torrent::resume_clear_progress(*download->download(), download->download()->bencode()->get_key("libtorrent_resume"));
 
   download->set_hash_failed(false);
+  download->variable()->set_value("hashing", type);
 
   if (download->is_open())
     throw torrent::internal_error("DownloadList::hash_clear(...) download still open.");
@@ -453,9 +484,7 @@ DownloadList::received_finished(Download* download) {
   if (control->variable()->get_value("check_hash")) {
     // Set some 'checking_finished_thingie' variable to make hash_done
     // trigger correctly, also so it can bork on missing data.
-
-    download->variable()->set("hashing", Download::variable_hashing_last);
-    hash_clear(download);
+    hash_queue(download, Download::variable_hashing_last);
 
   } else {
     confirm_finished(download);
@@ -474,17 +503,19 @@ DownloadList::confirm_finished(Download* download) {
 
   // Do this before the slots are called in case one of them closes
   // the download.
-  if (!download->is_active() && control->variable()->get_value("session_on_completion") == 1) {
-    download->download()->hash_resume_save();
+  if (!download->is_active() && control->variable()->get_value("session_on_completion") != 0) {
+    torrent::resume_save_progress(*download->download(), download->download()->bencode()->get_key("libtorrent_resume"));
     control->core()->download_store()->save(download);
   }
 
+  // Send the completed request before resuming so we don't reset the
+  // up/downloaded baseline.
+  download->download()->tracker_list().send_completed();
+
   std::for_each(slot_map_finished().begin(), slot_map_finished().end(), download_list_call(download));
 
   if (!download->is_active() && download->variable()->get_value("state") == 1)
     resume(download);
-
-  download->download()->tracker_list().send_completed();
 }
 
 }
