@@ -72,6 +72,8 @@ public:
   using base_type::empty;
   using base_type::size;
 
+  DownloadList() { }
+
   void                clear();
 
   void                session_save();
@@ -86,16 +88,18 @@ public:
 
   //void                save(Download* d);
 
-  void                open(Download* d);
+  bool                open(Download* d);
   void                open_throw(Download* d);
 
   void                close(Download* d);
   void                close_throw(Download* d);
 
-  void                start(Download* d);
-  void                stop(Download* d);
+  void                start_normal(Download* d);
+  bool                start_try(Download* d);
+
+  void                stop_normal(Download* d);
+  bool                stop_try(Download* d);
 
-  // These do not change the rtorrent:state.
   void                resume(Download* d);
   void                pause(Download* d);
 
@@ -168,8 +172,11 @@ public:
   static void         erase_key(slot_map& sm, const std::string& key)   { sm.erase(key); }
 
 private:
+  DownloadList(const DownloadList&);
+  void operator = (const DownloadList&);
+
   void                hash_done(Download* d);
-  void                hash_clear(Download* d);
+  void                hash_queue(Download* d, int type);
 
   inline void         check_contains(Download* d);
 
