@@ -45,12 +45,14 @@
 #include <rak/address_info.h>
 #include <rak/error_number.h>
 #include <rak/regex.h>
+#include <rak/path.h>
 #include <rak/string_manip.h>
 #include <sigc++/bind.h>
 #include <sigc++/hide.h>
 #include <torrent/object.h>
 #include <torrent/connection_manager.h>
 #include <torrent/exceptions.h>
+#include <torrent/resume.h>
 #include <torrent/tracker_list.h>
 
 #include "utils/variable_map.h"
@@ -103,14 +105,19 @@ receive_tracker_dump(const std::string& url, const char* data, size_t size) {
 }
 
 // Hmm... find some better place for all this.
-static void
-delete_tied(Download* d) {
-  const std::string tie = d->variable()->get("tied_to_file").as_string();
+void
+Manager::delete_tied(Download* d) {
+  const std::string& tie = d->variable()->get_string("tied_to_file");
 
   // This should be configurable, need to wait for the variable
   // thingie to be implemented.
-  if (!tie.empty())
-    ::unlink(tie.c_str());
+  if (tie.empty())
+    return;
+
+  if (::unlink(rak::path_expand(tie).c_str()) == -1)
+    push_log("Could not unlink tied file: " + std::string(rak::error_number::current().c_str()));
+
+  d->variable()->set("tied_to_file", std::string());
 }
 
 Manager::Manager() :
@@ -169,7 +176,7 @@ Manager::initialize_second() {
   m_downloadList->slot_map_insert()["1_connect_storage_log"]  = sigc::bind(sigc::ptr_fun(&connect_signal_storage_log), sigc::mem_fun(m_logComplete, &Log::push_front));
   m_downloadList->slot_map_insert()["1_connect_tracker_dump"] = sigc::bind(sigc::ptr_fun(&connect_signal_tracker_dump), sigc::ptr_fun(&receive_tracker_dump));
 
-  m_downloadList->slot_map_erase()["1_delete_tied"] = sigc::ptr_fun(&delete_tied);
+  m_downloadList->slot_map_erase()["1_delete_tied"] = sigc::mem_fun(this, &Manager::delete_tied);
 }
 
 void
@@ -211,7 +218,7 @@ Manager::listen_open() {
     int boundary = m_portFirst + random() % (m_portLast - m_portFirst + 1);
 
     if (torrent::connection_manager()->listen_open(boundary, m_portLast) ||
-	torrent::connection_manager()->listen_open(m_portFirst, boundary))
+        torrent::connection_manager()->listen_open(m_portFirst, boundary))
       return;
 
   } else {
@@ -316,8 +323,11 @@ path_expand(std::vector<std::string>* paths, const std::string& pattern) {
   if ((*first).empty()) {
     currentCache.push_back(utils::Directory("/"));
     ++first;
+  } else if (rak::trim(*first) == "~") {
+    currentCache.push_back(utils::Directory("~"));
+    ++first;
   } else {
-    currentCache.push_back(utils::Directory("./"));
+    currentCache.push_back(utils::Directory("."));
   }
 
   // Might be an idea to use depth-first search instead.
@@ -349,17 +359,17 @@ path_expand(std::vector<std::string>* paths, const std::string& pattern) {
 void
 Manager::try_create_download_expand(const std::string& uri, bool start, bool printLog, bool tied) {
   std::vector<std::string> paths;
-  paths.reserve(32);
+  paths.reserve(256);
 
   path_expand(&paths, uri);
 
   if (tied)
     for (std::vector<std::string>::iterator itr = paths.begin(); itr != paths.end(); )
-      if (std::find_if(m_downloadList->begin(), m_downloadList->end(),
-		       rak::equal(*itr, rak::bind2nd(std::mem_fun(&Download::variable_string), "tied_to_file"))) != m_downloadList->end())
-	itr = paths.erase(itr);
+      if (std::find_if(m_downloadList->begin(), m_downloadList->end(), rak::equal(*itr, rak::bind2nd(std::mem_fun(&Download::variable_string), "tied_to_file")))
+          != m_downloadList->end())
+        itr = paths.erase(itr);
       else
-	itr++;
+        itr++;
 
   if (!paths.empty())
     for (std::vector<std::string>::iterator itr = paths.begin(); itr != paths.end(); ++itr)
@@ -374,22 +384,60 @@ Manager::try_create_download_expand(const std::string& uri, bool start, bool pri
 // hashing view and starts hashing if nessesary.
 void
 Manager::receive_hashing_changed() {
-  if (m_hashingView->empty_visible() ||
-      std::find_if(m_hashingView->begin_visible(), m_hashingView->end_visible(), std::mem_fun(&Download::is_hash_checking)) != m_hashingView->end_visible())
-    return;
-
-  for (View::iterator itr = m_hashingView->begin_visible(), last = m_hashingView->end_visible(); itr != last; ++itr) {
-
+  bool foundHashing = false;
+  
+  // Try quick hashing all those with hashing == initial, set them to
+  // something else when failed.
+  for (View::iterator itr = m_hashingView->begin_visible(), last = m_hashingView->end_visible(); itr != last; ++itr)
     try {
 
+      if ((*itr)->is_hash_checking()) {
+        foundHashing = true;
+        continue;
+      }
+
       if ((*itr)->is_hash_checked())
-	throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked.");
+        throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
   
-      if ((*itr)->is_hash_failed())
-	continue;
+      if ((*itr)->is_hash_failed() || (*itr)->variable()->get_value("hashing") != Download::variable_hashing_initial)
+        continue;
 
       m_downloadList->open_throw(*itr);
-      (*itr)->download()->hash_check();
+      torrent::resume_load_progress(*(*itr)->download(), (*itr)->download()->bencode()->get_key("libtorrent_resume"));
+
+      if ((*itr)->download()->hash_check(true)) {
+        // Set this to exit the function so that the delayed hash done
+        // signal gets triggered before we try any real hashing.
+        foundHashing = true;
+
+      } else {
+        // Temporary hack.
+        (*itr)->download()->hash_stop();
+      }
+
+      (*itr)->variable()->set_value("hashing", Download::variable_hashing_rehash);
+
+    } catch (torrent::local_error& e) {
+      (*itr)->set_hash_failed(true);
+      push_log(e.what());
+    }
+
+  if (foundHashing)
+    return;
+
+  for (View::iterator itr = m_hashingView->begin_visible(), last = m_hashingView->end_visible(); itr != last; ++itr)
+    try {
+
+      if ((*itr)->is_hash_checked() || (*itr)->is_hash_checking())
+        throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
+  
+      if ((*itr)->is_hash_failed())
+        continue;
+
+      m_downloadList->open_throw(*itr);
+      torrent::resume_load_progress(*(*itr)->download(), (*itr)->download()->bencode()->get_key("libtorrent_resume"));
+
+      (*itr)->download()->hash_check(false);
 
       return;
 
@@ -397,7 +445,6 @@ Manager::receive_hashing_changed() {
       (*itr)->set_hash_failed(true);
       push_log(e.what());
     }
-  }
 }
 
 }
