@@ -65,7 +65,7 @@ PollManagerEPoll::poll(rak::timer timeout) {
   // Add 1ms to ensure we don't idle loop due to the lack of
   // resolution.
   torrent::perform();
-  timeout = std::min(timeout, rak::timer(torrent::next_timeout()));
+  timeout = std::min(timeout, rak::timer(torrent::next_timeout())) + 1000;
 
   if (m_httpStack.is_busy()) {
     // When we're using libcurl we need to use select, but as this is
@@ -82,7 +82,7 @@ PollManagerEPoll::poll(rak::timer timeout) {
     FD_SET(static_cast<torrent::PollEPoll*>(m_poll)->file_descriptor(), m_readSet);
 
     unsigned int maxFd = std::max((unsigned int)static_cast<torrent::PollEPoll*>(m_poll)->file_descriptor(),
-				  m_httpStack.fdset(m_readSet, m_writeSet, m_errorSet));
+                                  m_httpStack.fdset(m_readSet, m_writeSet, m_errorSet));
 
     timeval t = timeout.tval();
 
