@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 
 #include "config.h"
 
+#include <unistd.h>
+#include <sys/ioctl.h>
+#include <sys/termios.h>
+
 #include "canvas.h"
 
 namespace display {
@@ -74,4 +78,14 @@ Canvas::cleanup() {
   endwin();
 }
 
+std::pair<int, int>
+Canvas::term_size() {
+  struct winsize ws;
+
+  if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) == 0)
+    return std::pair<int, int>(ws.ws_col, ws.ws_row);
+  else
+    return std::pair<int, int>(80, 24);
+}
+
 }
