@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -64,6 +64,8 @@ public:
   Manager();
   ~Manager();
 
+  void                force_redraw();
+
   iterator            insert(iterator pos, Window* w);
   iterator            erase(iterator pos);
   iterator            erase(Window* w);
@@ -74,12 +76,12 @@ public:
   void                unschedule(Window* w);
 
   void                adjust_layout();
-
-private:
   void                receive_update();
 
+private:
   void                schedule_update();
 
+  bool                m_forceRedraw;
   rak::timer          m_timeLastUpdate;
 
   rak::priority_queue_default m_scheduler;
