@@ -89,7 +89,7 @@ print_ddhhmm(char* first, char* last, time_t t) {
   if (t / (24 * 3600) < 100)
     return print_buffer(first, last, "%2id %2i:%02i", (int)t / (24 * 3600), ((int)t / 3600) % 24, ((int)t / 60) % 60);
   else
-    return print_buffer(first, last, "--:--:--");
+    return print_buffer(first, last, "--d --:--");
 }
 
 char*
@@ -123,22 +123,24 @@ print_download_title(char* first, char* last, core::Download* d) {
 
 char*
 print_download_info(char* first, char* last, core::Download* d) {
-  first = print_buffer(first, last, "Torrent: ");
-
   if (!d->download()->is_open())
-    first = print_buffer(first, last, "closed            ");
+    first = print_buffer(first, last, "[CLOSED]  ");
+  else if (!d->download()->is_active())
+    first = print_buffer(first, last, "[OPEN]    ");
+  else
+    first = print_buffer(first, last, "          ");
 
-  else if (d->is_done())
+  if (d->is_done())
     first = print_buffer(first, last, "done %10.1f MB", (double)d->download()->bytes_total() / (double)(1 << 20));
   else
     first = print_buffer(first, last, "%6.1f / %6.1f MB",
-		       (double)d->download()->bytes_done() / (double)(1 << 20),
-		       (double)d->download()->bytes_total() / (double)(1 << 20));
+                         (double)d->download()->bytes_done() / (double)(1 << 20),
+                         (double)d->download()->bytes_total() / (double)(1 << 20));
   
   first = print_buffer(first, last, " Rate: %5.1f / %5.1f KB Uploaded: %7.1f MB",
-		     (double)d->download()->up_rate()->rate() / (1 << 10),
-		     (double)d->download()->down_rate()->rate() / (1 << 10),
-		     (double)d->download()->up_rate()->total() / (1 << 20));
+                       (double)d->download()->up_rate()->rate() / (1 << 10),
+                       (double)d->download()->down_rate()->rate() / (1 << 10),
+                       (double)d->download()->up_rate()->total() / (1 << 20));
 
   if (d->download()->is_active() && !d->is_done()) {
     first = print_buffer(first, last, " ");
@@ -147,11 +149,18 @@ print_download_info(char* first, char* last, core::Download* d) {
     first = print_buffer(first, last, " ");
     first = print_download_time_left(first, last, d);
   } else {
-    first = print_buffer(first, last, "               ");
+    first = print_buffer(first, last, "                ");
   }
 
+  first = print_buffer(first, last, " [%c%c R: %3.2f",
+                       d->variable()->get_string("tied_to_file").empty() ? ' ' : 'T',
+                       d->variable()->get_value("ignore_commands") == 0 ? ' ' : 'I',
+                       d->download()->bytes_done() > 0 ? (double)(100 * d->download()->up_rate()->total() / d->download()->bytes_done()) / 100 : 0.0);
+
   if (d->priority() != 2)
-    first = print_buffer(first, last, " [%s]", core::Download::priority_to_string(d->priority()));
+    first = print_buffer(first, last, " %s]", core::Download::priority_to_string(d->priority()));
+  else
+    first = print_buffer(first, last, "]");
 
   if (first > last)
     throw torrent::client_error("print_download_info(...) wrote past end of the buffer.");
@@ -166,13 +175,13 @@ print_download_status(char* first, char* last, core::Download* d) {
 
   if (d->is_hash_checking()) {
     first = print_buffer(first, last, "Checking hash [%2i%%]",
-		       (d->download()->chunks_hashed() * 100) / d->download()->chunks_total());
+                         (d->download()->chunks_hashed() * 100) / d->download()->chunks_total());
 
   } else if (d->tracker_list()->is_busy() && d->tracker_list()->focus() < d->tracker_list()->size()) {
     torrent::TrackerList* tl = d->tracker_list();
 
     first = print_buffer(first, last, "Tracker[%i:%i]: Connecting to %s",
-			 tl->get(tl->focus()).group(), tl->focus(), tl->get(tl->focus()).url().c_str());
+                         tl->get(tl->focus()).group(), tl->focus(), tl->get(tl->focus()).url().c_str());
 
   } else if (!d->message().empty()) {
     first = print_buffer(first, last, "%s", d->message().c_str());
@@ -192,7 +201,7 @@ print_download_time_left(char* first, char* last, core::Download* d) {
   uint32_t rate = d->download()->down_rate()->rate();
 
   if (rate < 512)
-    return print_buffer(first, last, "--:--:--");
+    return print_buffer(first, last, "--d --:--");
   
   time_t remaining = (d->download()->bytes_total() - d->download()->bytes_done()) / (rate & ~(uint32_t)(512 - 1));
 
@@ -221,8 +230,8 @@ print_status_info(char* first, char* last) {
     first = print_buffer(first, last, "/%3i KB]", torrent::down_throttle() / 1024);
   
   first = print_buffer(first, last, " [Rate %5.1f/%5.1f KB]",
-		       (double)torrent::up_rate()->rate() / 1024.0,
-		       (double)torrent::down_rate()->rate() / 1024.0);
+                       (double)torrent::up_rate()->rate() / 1024.0,
+                       (double)torrent::down_rate()->rate() / 1024.0);
 
   first = print_buffer(first, last, " [Port: %i]", (unsigned int)torrent::connection_manager()->listen_port());
 
@@ -247,17 +256,17 @@ print_status_info(char* first, char* last) {
 char*
 print_status_extra(char* first, char* last, __UNUSED Control* c) {
   first = print_buffer(first, last, " [U %i/%i]",
-		       torrent::currently_unchoked(),
-		       torrent::max_unchoked());
+                       torrent::currently_unchoked(),
+                       torrent::max_unchoked());
 
   first = print_buffer(first, last, " [S %i/%i/%i]",
-		       torrent::total_handshakes(),
-		       torrent::open_sockets(),
-		       torrent::max_open_sockets());
-		       
+                       torrent::total_handshakes(),
+                       torrent::open_sockets(),
+                       torrent::max_open_sockets());
+                       
   first = print_buffer(first, last, " [F %i/%i]",
-		       torrent::open_files(),
-		       torrent::max_open_files());
+                       torrent::open_files(),
+                       torrent::max_open_files());
 
   return first;
 }
