@@ -38,8 +38,12 @@
 
 #include <cmath>
 #include <stdexcept>
+#include <rak/functional.h>
 #include <rak/string_manip.h>
 #include <torrent/bitfield.h>
+#include <torrent/transfer_list.h>
+#include <torrent/block.h>
+#include <torrent/block_list.h>
 
 #include "core/download.h"
 
@@ -47,9 +51,10 @@
 
 namespace display {
 
-WindowDownloadChunksSeen::WindowDownloadChunksSeen(core::Download* d) :
+WindowDownloadChunksSeen::WindowDownloadChunksSeen(core::Download* d, unsigned int *focus) :
   Window(new Canvas, true),
-  m_download(d) {
+  m_download(d),
+  m_focus(focus) {
 }
 
 void
@@ -62,46 +67,78 @@ WindowDownloadChunksSeen::redraw() {
     return;
 
   m_canvas->print(2, 0, "Chunks seen: [C/A/D %i/%i/%.2f]",
-		  (int)m_download->download()->peers_complete(),
-		  (int)m_download->download()->peers_accounted(),
-		  std::floor(m_download->distributed_copies() * 100.0f) / 100.0f);
+                  (int)m_download->download()->peers_complete(),
+                  (int)m_download->download()->peers_accounted(),
+                  std::floor(m_download->distributed_copies() * 100.0f) / 100.0f);
 
   const uint8_t* seen = m_download->download()->chunks_seen();
 
-  if (seen == NULL) {
+  if (seen == NULL || m_download->download()->bitfield()->empty()) {
     m_canvas->print(2, 2, "Not available.");
     return;
   }
 
-  const uint8_t* chunk = seen;
+  if (!m_download->is_done()) { 
+    m_canvas->print(36, 0, "X downloaded    missing    queued    downloading");
+    m_canvas->print_char(50, 0, 'X' | A_BOLD);
+    m_canvas->print_char(61, 0, 'X' | A_BOLD | A_UNDERLINE);
+    m_canvas->print_char(71, 0, 'X' | A_REVERSE);
+  }
+
+  *m_focus = std::min(*m_focus, max_focus());
+
+  const uint8_t* chunk = seen + *m_focus * chunks_per_row();
   const uint8_t* last = seen + m_download->download()->chunks_total();
 
   const torrent::Bitfield* bitfield = m_download->download()->bitfield();
+  const torrent::TransferList* transfers = m_download->download()->transfer_list();
+  std::vector<torrent::BlockList*> transferChunks(transfers->size(), 0);
+
+  std::copy(transfers->begin(), transfers->end(), transferChunks.begin());
+  std::sort(transferChunks.begin(), transferChunks.end(), rak::less2(std::mem_fun(&torrent::BlockList::index), std::mem_fun(&torrent::BlockList::index)));
+
+  std::vector<torrent::BlockList*>::const_iterator itrTransfer = transferChunks.begin();
+
+  while (itrTransfer != transferChunks.end() && (uint32_t)(chunk - seen) > (*itrTransfer)->index())
+    itrTransfer++;
 
   for (int y = 1; y < m_canvas->get_height() && chunk < last; ++y) {
-    m_canvas->print(0, y, "%5d ", (int)(chunk - seen));
+    m_canvas->print(0, y, "%5u ", (int)(chunk - seen));
 
     while (chunk < last) {
       chtype attr;
 
-      if (bitfield->get(chunk - seen))
-	attr = A_NORMAL;
-//    else if (foo->is_downloading(chunk - seen))
-//	attr = A_UNDERLINE;
-      else
-	attr = A_BOLD;
+      if (bitfield->get(chunk - seen)) {
+        attr = A_NORMAL;
+      } else if (itrTransfer != transferChunks.end() && (uint32_t)(chunk - seen) == (*itrTransfer)->index()) {
+        if (std::find_if((*itrTransfer)->begin(), (*itrTransfer)->end(), std::mem_fun_ref(&torrent::Block::is_transfering)) != (*itrTransfer)->end())
+          attr = A_REVERSE;
+        else
+          attr = A_BOLD | A_UNDERLINE;
+        itrTransfer++;
+      } else {
+        attr = A_BOLD;
+      }
 
       m_canvas->print_char(attr | rak::value_to_hexchar<0>(std::min<uint8_t>(*chunk, 0xF)));
       chunk++;
 
       if ((chunk - seen) % 10 == 0) {
-	if (m_canvas->get_x() + 12 > m_canvas->get_width())
-	  break;
+        if (m_canvas->get_x() + 12 > m_canvas->get_width())
+          break;
 
-	m_canvas->print_char(' ');
+        m_canvas->print_char(' ');
       }
     }
   }
 }
 
+unsigned int
+WindowDownloadChunksSeen::rows() const {
+  if (m_canvas->get_width() < 18)
+    return 0;
+
+  return (m_download->download()->chunks_total() + chunks_per_row() - 1) / chunks_per_row();
+}
+
 }
