@@ -38,6 +38,7 @@
 
 #include <stdexcept>
 #include <rak/socket_address.h>
+#include <torrent/block_transfer.h>
 #include <torrent/rate.h>
 #include <torrent/piece.h>
 
@@ -73,7 +74,8 @@ WindowPeerList::redraw() {
   m_canvas->print(x, y, "QS");      x += 6;
   m_canvas->print(x, y, "DONE");    x += 6;
   m_canvas->print(x, y, "REQ");     x += 6;
-  m_canvas->print(x, y, "SNUB");
+  m_canvas->print(x, y, "SNUB");    x += 6;
+  m_canvas->print(x, y, "FAILED");
 
   ++y;
 
@@ -83,9 +85,9 @@ WindowPeerList::redraw() {
   typedef std::pair<PList::iterator, PList::iterator> Range;
 
   Range range = rak::advance_bidirectional(m_list->begin(),
-					   *m_focus != m_list->end() ? *m_focus : m_list->begin(),
-					   m_list->end(),
-					   m_canvas->get_height() - y);
+                                           *m_focus != m_list->end() ? *m_focus : m_list->begin(),
+                                           m_list->end(),
+                                           m_canvas->get_height() - y);
 
   if (m_download->download()->chunks_total() <= 0)
     throw std::logic_error("WindowPeerList::redraw() m_slotChunksTotal() returned invalid value");
@@ -96,8 +98,8 @@ WindowPeerList::redraw() {
     x = 0;
 
     m_canvas->print(x, y, "%c %s",
-		    range.first == *m_focus ? '*' : ' ',
-		    rak::socket_address::cast_from(p.address())->address_str().c_str());
+                    range.first == *m_focus ? '*' : ' ',
+                    rak::socket_address::cast_from(p.address())->address_str().c_str());
     x += 18;
 
     m_canvas->print(x, y, "%.1f", (double)p.up_rate()->rate() / 1024); x += 7;
@@ -105,30 +107,35 @@ WindowPeerList::redraw() {
     m_canvas->print(x, y, "%.1f", (double)p.peer_rate()->rate() / 1024); x += 7;
 
     m_canvas->print(x, y, "%c/%c%c/%c%c",
-		    p.is_incoming() ? 'r' : 'l',
-		    p.is_remote_choked() ? 'c' : 'u',
-		    p.is_remote_interested() ? 'i' : 'n',
-		    p.is_local_choked() ? 'c' : 'u',
-		    p.is_local_interested() ? 'i' : 'n');
+                    p.is_incoming() ? 'r' : 'l',
+                    p.is_remote_choked() ? 'c' : 'u',
+                    p.is_remote_interested() ? 'i' : 'n',
+                    p.is_local_choked() ? 'c' : 'u',
+                    p.is_local_interested() ? 'i' : 'n');
     x += 9;
 
-    m_canvas->print(x, y, "%i/%i",
-		    p.outgoing_queue_size(),
-		    p.incoming_queue_size());
+    m_canvas->print(x, y, "%i/%i", p.outgoing_queue_size(), p.incoming_queue_size());
     x += 6;
 
     m_canvas->print(x, y, "%3i", done_percentage(*range.first));
     x += 6;
 
-    if (p.incoming_queue_size())
-      m_canvas->print(x, y, "%i", p.incoming_queue(0)->index());
+    const torrent::BlockTransfer* transfer = p.transfer();
+
+    if (transfer != NULL)
+      m_canvas->print(x, y, "%i", transfer->index());
 
     x += 6;
 
     if (p.is_snubbed())
       m_canvas->print(x, y, "*");
 
-    x += 5;
+    x += 6;
+
+    if (p.failed_counter() != 0)
+      m_canvas->print(x, y, "%u", p.failed_counter());
+
+    x += 7;
 
     char buf[128];
     control->client_info()->print(buf, buf + 128, p.id().c_str());
