@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent library
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,7 +40,11 @@
 #include <rak/timer.h>
 #include <rak/priority_queue_default.h>
 
+#include "control.h"
+
 extern rak::priority_queue_default taskScheduler;
 extern rak::timer                  cachedTime;
 
+extern Control*                    control;
+
 #endif
