@@ -41,17 +41,22 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <rak/file_stat.h>
+#include <rak/fs_stat.h>
 #include <rak/functional.h>
 #include <rak/path.h>
 #include <rak/string_manip.h>
 #include <torrent/object.h>
 #include <torrent/connection_manager.h>
 #include <torrent/exceptions.h>
+#include <torrent/file.h>
+#include <torrent/path.h>
+#include <torrent/rate.h>
 #include <torrent/torrent.h>
 #include <torrent/tracker.h>
 #include <torrent/tracker_list.h>
 
 #include "core/download.h"
+#include "core/download_list.h"
 #include "core/download_store.h"
 #include "core/manager.h"
 #include "core/scheduler.h"
@@ -66,6 +71,11 @@
 #include "option_handler_rules.h"
 #include "command_scheduler.h"
 
+namespace core {
+  extern void
+  path_expand(std::vector<std::string>* paths, const std::string& pattern);
+}
+
 void
 apply_hash_read_ahead(__UNUSED Control* m, int arg) {
   torrent::set_hash_read_ahead(arg << 20);
@@ -92,43 +102,89 @@ apply_load(Control* m, const std::string& arg) {
   m->core()->try_create_download_expand(arg, false, false, true);
 }
 
+void
+apply_load_verbose(Control* m, const std::string& arg) {
+  m->core()->try_create_download_expand(arg, false, true, true);
+}
+
 void
 apply_load_start(Control* m, const std::string& arg) {
   m->core()->try_create_download_expand(arg, true, false, true);
 }
 
 void
-apply_stop_untied(Control* m) {
-  core::Manager::DListItr itr = m->core()->download_list()->begin();
+apply_load_start_verbose(Control* m, const std::string& arg) {
+  m->core()->try_create_download_expand(arg, true, true, true);
+}
+
+void
+apply_start_tied(Control* m) {
+  for (core::DownloadList::iterator itr = m->core()->download_list()->begin(); itr != m->core()->download_list()->end(); ++itr) {
+    if ((*itr)->variable_value("state") == 1)
+      continue;
 
-  while ((itr = std::find_if(itr, m->core()->download_list()->end(),
-			     rak::on(rak::bind2nd(std::mem_fun(&core::Download::variable_string), "tied_to_file"),
-				     std::not1(std::mem_fun_ref(&std::string::empty)))))
-	 != m->core()->download_list()->end()) {
     rak::file_stat fs;
+    const std::string& tiedToFile = (*itr)->variable_string("tied_to_file");
 
-    if (!fs.update(rak::path_expand((*itr)->variable_string("tied_to_file")))) {
-      (*itr)->variable()->set("tied_to_file", std::string());
-      m->core()->download_list()->stop(*itr);
-    }
+    if (!tiedToFile.empty() && fs.update(rak::path_expand(tiedToFile)))
+      m->core()->download_list()->start_try(*itr);
+  }
+}
 
-    ++itr;
+void
+apply_stop_untied(Control* m) {
+  for (core::DownloadList::iterator itr = m->core()->download_list()->begin(); itr != m->core()->download_list()->end(); ++itr) {
+    if ((*itr)->variable_value("state") == 0)
+      continue;
+
+    rak::file_stat fs;
+    const std::string& tiedToFile = (*itr)->variable_string("tied_to_file");
+
+    if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
+      m->core()->download_list()->stop_try(*itr);
   }
 }
 
 void
 apply_close_untied(Control* m) {
+  for (core::DownloadList::iterator itr = m->core()->download_list()->begin(); itr != m->core()->download_list()->end(); ++itr) {
+    rak::file_stat fs;
+    const std::string& tiedToFile = (*itr)->variable_string("tied_to_file");
+
+    if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)) && m->core()->download_list()->stop_try(*itr))
+      m->core()->download_list()->close(*itr);
+  }
+}
+
+void
+apply_remove_untied(Control* m) {
+  for (core::DownloadList::iterator itr = m->core()->download_list()->begin(); itr != m->core()->download_list()->end(); ) {
+    rak::file_stat fs;
+    const std::string& tiedToFile = (*itr)->variable_string("tied_to_file");
+
+    if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)) && m->core()->download_list()->stop_try(*itr))
+      itr = m->core()->download_list()->erase(itr);
+    else
+      ++itr;
+  }
+}
+
+void
+apply_close_low_diskspace(Control* m, int64_t arg) {
   core::Manager::DListItr itr = m->core()->download_list()->begin();
 
-  while ((itr = std::find_if(itr, m->core()->download_list()->end(),
-			     rak::on(rak::bind2nd(std::mem_fun(&core::Download::variable_string), "tied_to_file"),
-				     std::not1(std::mem_fun_ref(&std::string::empty)))))
-	 != m->core()->download_list()->end()) {
-    rak::file_stat fs;
+  while ((itr = std::find_if(itr, m->core()->download_list()->end(), std::mem_fun(&core::Download::is_downloading))) != m->core()->download_list()->end()) {
+    rak::fs_stat stat;
+    std::string path = (*itr)->file_list()->root_dir() + (*itr)->file_list()->get(0).path()->as_string();
 
-    if (!fs.update(rak::path_expand((*itr)->variable_string("tied_to_file")))) {
-      (*itr)->variable()->set("tied_to_file", std::string());
+    if (!stat.update(path)) {
+      m->core()->push_log(std::string("Cannot read free diskspace: ") + strerror(errno) + " for " + path);
+
+    } else if (stat.bytes_avail() < arg) {
       m->core()->download_list()->close(*itr);
+
+      (*itr)->set_hash_failed(true);
+      (*itr)->set_message(std::string("Low diskspace"));
     }
 
     ++itr;
@@ -136,23 +192,31 @@ apply_close_untied(Control* m) {
 }
 
 void
-apply_remove_untied(Control* m) {
+apply_stop_on_ratio(Control* m, const std::string& arg) {
+  int64_t minRatio = 0;  // first argument:  minimum ratio to reach
+  int64_t minUpload = 0; // second argument: minimum upload amount to reach [optional]
+  int64_t maxRatio = 0;  // third argument:  maximum ratio to reach [optional]
+
+  rak::split_iterator_t<std::string> sitr = rak::split_iterator(arg, ',');
+
+  utils::Variable::string_to_value_unit(rak::trim(*sitr).c_str(), &minRatio, 0, 1);
+
+  if (++sitr != rak::split_iterator(arg))
+    utils::Variable::string_to_value_unit(rak::trim(*sitr).c_str(), &minUpload, 0, 1);
+
+  if (++sitr != rak::split_iterator(arg))
+    utils::Variable::string_to_value_unit(rak::trim(*sitr).c_str(), &maxRatio, 0, 1);
+
   core::Manager::DListItr itr = m->core()->download_list()->begin();
 
-  while ((itr = std::find_if(itr, m->core()->download_list()->end(),
-			     rak::on(rak::bind2nd(std::mem_fun(&core::Download::variable_string), "tied_to_file"),
-				     std::not1(std::mem_fun_ref(&std::string::empty)))))
-	 != m->core()->download_list()->end()) {
-    rak::file_stat fs;
+  while ((itr = std::find_if(itr, m->core()->download_list()->end(), std::mem_fun(&core::Download::is_seeding))) != m->core()->download_list()->end()) {
+    int64_t totalUpload = (*itr)->download()->up_rate()->total();
+    int64_t totalDone = (*itr)->download()->bytes_done();
 
-    if (!fs.update(rak::path_expand((*itr)->variable_string("tied_to_file")))) {
-      (*itr)->variable()->set("tied_to_file", std::string());
-      m->core()->download_list()->stop(*itr);
-      itr = m->core()->download_list()->erase(itr);
+    if ((totalUpload >= minUpload && totalUpload * 100 >= totalDone * minRatio) || (maxRatio > 0 && totalUpload * 100 > totalDone * maxRatio))
+        m->core()->download_list()->stop_try(*itr);
 
-    } else {
-      ++itr;
-    }
+    ++itr;
   }
 }
 
@@ -171,9 +235,9 @@ apply_enable_trackers(Control* m, __UNUSED const std::string& arg) {
 
     for (int i = 0, last = tl.size(); i < last; ++i)
       if (state)
-	tl.get(i).enable();
+        tl.get(i).enable();
       else
-	tl.get(i).disable();
+        tl.get(i).disable();
 
     if (state && !control->variable()->get_value("use_udp_trackers"))
       (*itr)->enable_udp_trackers(false);
@@ -336,7 +400,7 @@ initialize_option_handler(Control* c) {
   variables->insert("tracker_dump",          new utils::VariableAny(std::string()));
 
   variables->insert("session",               new utils::VariableStringSlot(rak::mem_fn(control->core()->download_store(), &core::DownloadStore::path),
-									   rak::mem_fn(control->core()->download_store(), &core::DownloadStore::set_path)));
+                                                                           rak::mem_fn(control->core()->download_store(), &core::DownloadStore::set_path)));
   variables->insert("session_lock",          new utils::VariableBool(true));
   variables->insert("session_on_completion", new utils::VariableBool(true));
   variables->insert("session_save",          new utils::VariableVoidSlot(rak::mem_fn(c->core()->download_list(), &core::DownloadList::session_save)));
@@ -349,21 +413,25 @@ initialize_option_handler(Control* c) {
   variables->insert("tos",                   new utils::VariableStringSlot(rak::value_fn(std::string()), rak::ptr_fn(&apply_tos)));
 
   variables->insert("bind",                  new utils::VariableStringSlot(rak::mem_fn(control->core(), &core::Manager::bind_address),
-									   rak::mem_fn(control->core(), &core::Manager::set_bind_address)));
+                                                                           rak::mem_fn(control->core(), &core::Manager::set_bind_address)));
   variables->insert("ip",                    new utils::VariableStringSlot(rak::mem_fn(control->core(), &core::Manager::local_address),
-									   rak::mem_fn(control->core(), &core::Manager::set_local_address)));
+                                                                           rak::mem_fn(control->core(), &core::Manager::set_local_address)));
 
   variables->insert("http_proxy",            new utils::VariableStringSlot(rak::mem_fn(c->core()->get_poll_manager()->get_http_stack(), &core::CurlStack::http_proxy),
-									   rak::mem_fn(c->core()->get_poll_manager()->get_http_stack(), &core::CurlStack::set_http_proxy)));
+                                                                           rak::mem_fn(c->core()->get_poll_manager()->get_http_stack(), &core::CurlStack::set_http_proxy)));
 
   variables->insert("min_peers",             new utils::VariableValue(40));
   variables->insert("max_peers",             new utils::VariableValue(100));
   variables->insert("max_uploads",           new utils::VariableValue(15));
+  variables->insert("max_chunks_queued",     new utils::VariableValue(0));
+
+  variables->insert("timeout_sync",          new utils::VariableValue(0));
+  variables->insert("timeout_safe_sync",     new utils::VariableValue(0));
 
   variables->insert("download_rate",         new utils::VariableValueSlot(rak::ptr_fn(&torrent::down_throttle), rak::mem_fn(control->ui(), &ui::Root::set_down_throttle_i64),
-									  0, (1 << 10)));
+                                                                          0, (1 << 10)));
   variables->insert("upload_rate",           new utils::VariableValueSlot(rak::ptr_fn(&torrent::up_throttle), rak::mem_fn(control->ui(), &ui::Root::set_up_throttle_i64),
-									  0, (1 << 10)));
+                                                                          0, (1 << 10)));
 
   variables->insert("hash_max_tries",        new utils::VariableValueSlot(rak::ptr_fn(&torrent::hash_max_tries), rak::ptr_fn(&torrent::set_hash_max_tries)));
   variables->insert("max_open_files",        new utils::VariableValueSlot(rak::ptr_fn(&torrent::max_open_files), rak::ptr_fn(&torrent::set_max_open_files)));
@@ -381,17 +449,19 @@ initialize_option_handler(Control* c) {
   variables->insert("view_sort_new",         new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_view_sort_new, c)));
   variables->insert("view_sort_current",     new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_view_sort_current, c)));
 
+  variables->insert("key_layout",            new utils::VariableAny(std::string("qwerty")));
+
   variables->insert("schedule",              new utils::VariableStringSlot(rak::value_fn(std::string()), rak::mem_fn(c->command_scheduler(), &CommandScheduler::parse)));
   variables->insert("schedule_remove",       new utils::VariableStringSlot(rak::value_fn(std::string()),
-									   rak::mem_fn<const std::string&>(c->command_scheduler(), &CommandScheduler::erase)));
+                                                                           rak::mem_fn<const std::string&>(c->command_scheduler(), &CommandScheduler::erase)));
 
   variables->insert("download_scheduler",    new utils::VariableVoidSlot(rak::mem_fn(control->scheduler(), &core::Scheduler::update)));
 
   variables->insert("send_buffer_size",      new utils::VariableValueSlot(rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::send_buffer_size),
-									  rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::set_send_buffer_size)));
+                                                                          rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::set_send_buffer_size)));
   
   variables->insert("receive_buffer_size",   new utils::VariableValueSlot(rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::receive_buffer_size),
-									  rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::set_receive_buffer_size)));
+                                                                          rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::set_receive_buffer_size)));
   
   variables->insert("port_range",            new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_port_range, c)));
 
@@ -400,14 +470,21 @@ initialize_option_handler(Control* c) {
 
   variables->insert("umask",                 new utils::VariableValueSlot(rak::mem_fn(control, &Control::umask), rak::mem_fn(control, &Control::set_umask), 8));
   variables->insert("working_directory",     new utils::VariableStringSlot(rak::mem_fn(control, &Control::working_directory),
-									   rak::mem_fn(control, &Control::set_working_directory)));
+                                                                           rak::mem_fn(control, &Control::set_working_directory)));
 
   variables->insert("load",                  new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_load, c)));
+  variables->insert("load_verbose",          new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_load_verbose, c)));
   variables->insert("load_start",            new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_load_start, c)));
+  variables->insert("load_start_verbose",    new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_load_start_verbose, c)));
+
+  variables->insert("start_tied",            new utils::VariableVoidSlot(rak::bind_ptr_fn(&apply_start_tied, c)));
   variables->insert("stop_untied",           new utils::VariableVoidSlot(rak::bind_ptr_fn(&apply_stop_untied, c)));
   variables->insert("close_untied",          new utils::VariableVoidSlot(rak::bind_ptr_fn(&apply_close_untied, c)));
   variables->insert("remove_untied",         new utils::VariableVoidSlot(rak::bind_ptr_fn(&apply_remove_untied, c)));
 
+  variables->insert("close_low_diskspace",   new utils::VariableValueSlot(rak::value_fn(int64_t()), rak::bind_ptr_fn(&apply_close_low_diskspace, c)));
+  variables->insert("stop_on_ratio",         new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_stop_on_ratio, c)));
+
   variables->insert("enable_trackers",       new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_enable_trackers, c)));
   variables->insert("encoding_list",         new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_encoding_list, c)));
 }
