@@ -77,9 +77,6 @@ OptionParser::process(int argc, char** argv) {
   int c;
   std::string optString = create_optstring();
 
-  optind = 0;
-  opterr = 0;
-
   while ((c = getopt(argc, argv, optString.c_str())) != -1)
     if (c == '?')
       throw std::runtime_error("Invalid/unknown option flag \"-" + std::string(1, (char)optopt) + "\". See rtorrent -h for more information.");
