@@ -200,7 +200,7 @@ DownloadList::receive_start_download() {
   if (m_view->focus() == m_view->end_visible())
     return;
 
-  m_control->core()->download_list()->start(*m_view->focus());
+  m_control->core()->download_list()->start_normal(*m_view->focus());
   m_view->set_last_changed();
 }
 
@@ -210,7 +210,7 @@ DownloadList::receive_stop_download() {
     return;
 
   if ((*m_view->focus())->variable()->get_value("state") == 1)
-    m_control->core()->download_list()->stop(*m_view->focus());
+    m_control->core()->download_list()->stop_normal(*m_view->focus());
   else
     m_control->core()->download_list()->erase(*m_view->focus());
 
@@ -222,6 +222,7 @@ DownloadList::receive_close_download() {
   if (m_view->focus() == m_view->end_visible())
     return;
 
+  m_control->core()->download_list()->stop_normal(*m_view->focus());
   m_control->core()->download_list()->close(*m_view->focus());
   m_view->set_last_changed();
 }
@@ -282,6 +283,35 @@ DownloadList::receive_check_hash() {
   m_control->core()->download_list()->check_hash(*m_view->focus());
 }
 
+void
+DownloadList::receive_ignore_ratio() {
+  if (m_view->focus() == m_view->end_visible())
+    return;
+
+  if ((*m_view->focus())->variable()->get_value("ignore_commands") != 0) {
+    (*m_view->focus())->variable()->set("ignore_commands", (int64_t)0);
+    m_control->core()->push_log("Torrent set to heed commands.");
+  } else {
+    (*m_view->focus())->variable()->set("ignore_commands", (int64_t)1);
+    m_control->core()->push_log("Torrent set to ignore commands");
+  }
+}
+
+void
+DownloadList::receive_clear_tied() {
+  if (m_view->focus() == m_view->end_visible())
+    return;
+
+  const std::string& tiedFile = (*m_view->focus())->variable()->get_string("tied_to_file");
+
+  if (!tiedFile.empty()) {
+    // Move this into core?
+    m_control->core()->delete_tied(*m_view->focus());
+
+    m_control->core()->push_log("Cleared tied to file association for download.");
+  }
+}
+
 void
 DownloadList::receive_view_input(Input type) {
   if (m_windowTextInput->get_active())
@@ -327,7 +357,7 @@ DownloadList::receive_exit_input(Input type) {
 
     case INPUT_CHANGE_DIRECTORY:
       if (m_view->focus() == m_view->end_visible())
-	throw torrent::input_error("No download in focus to change root directory.");
+        throw torrent::input_error("No download in focus to change root directory.");
 
       (*m_view->focus())->variable()->set("directory", rak::trim(m_windowTextInput->get_input()->str()));
       m_control->core()->push_log("New root dir \"" + (*m_view->focus())->variable()->get_string("directory") + "\" for torrent.");
@@ -411,6 +441,8 @@ DownloadList::setup_keys() {
   (*m_bindings)['\x12']        = sigc::mem_fun(*this, &DownloadList::receive_check_hash);
   (*m_bindings)['+']           = sigc::mem_fun(*this, &DownloadList::receive_next_priority);
   (*m_bindings)['-']           = sigc::mem_fun(*this, &DownloadList::receive_prev_priority);
+  (*m_bindings)['I']           = sigc::mem_fun(*this, &DownloadList::receive_ignore_ratio);
+  (*m_bindings)['U']           = sigc::mem_fun(*this, &DownloadList::receive_clear_tied);
 
   (*m_bindings)['\x7f']        = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_view_input), INPUT_LOAD_DEFAULT);
   (*m_bindings)[KEY_BACKSPACE] = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_view_input), INPUT_LOAD_DEFAULT);
