@@ -130,6 +130,9 @@ private:
 
   void                receive_check_hash();
 
+  void                receive_ignore_ratio();
+  void                receive_clear_tied();
+
   void                receive_view_input(Input type);
   void                receive_exit_input(Input type);
 
