@@ -53,7 +53,7 @@ namespace ui {
 
 class ElementFileList : public ElementBase {
 public:
-  typedef torrent::File::Priority Priority;
+  typedef torrent::priority_t Priority;
   typedef display::WindowFileList  WFileList;
 
   ElementFileList(core::Download* d);
@@ -64,6 +64,8 @@ public:
 private:
   void                receive_next();
   void                receive_prev();
+  void                receive_pagenext();
+  void                receive_pageprev();
 
   void                receive_priority();
   void                receive_change_all();
